import { MigrationInterface, QueryRunner } from 'typeorm';

export class MainEntitySetup1620348350833 implements MigrationInterface {
  name = 'MainEntitySetup1620348350833';

  public async up(queryRunner: QueryRunner): Promise<void> {
    await queryRunner.query(`
            CREATE TABLE "targets" (
                "id" uuid NOT NULL DEFAULT uuid_generate_v4(),
                "ip_address" character varying NOT NULL,
                "domains" text array NOT NULL,
                "extra" json,
                "notes" character varying,
                "created_at" TIMESTAMP NOT NULL DEFAULT now(),
                "updated_at" TIMESTAMP NOT NULL DEFAULT now(),
                "deleted_at" TIMESTAMP,
                CONSTRAINT "UQ_97ea310b1099d492dca65c5714b" UNIQUE ("ip_address"),
                CONSTRAINT "PK_87084f49e9de9dd6a3e83906584" PRIMARY KEY ("id")
            );
            COMMENT ON COLUMN "targets"."ip_address" IS 'Target IP address';
            COMMENT ON COLUMN "targets"."domains" IS 'Domains directed to this address';
            COMMENT ON COLUMN "targets"."extra" IS 'Extra data from scanner';
            COMMENT ON COLUMN "targets"."notes" IS 'User notes for this target'
        `);
    await queryRunner.query(`
            CREATE TABLE "services" (
                "id" uuid NOT NULL DEFAULT uuid_generate_v4(),
                "name" character varying,
                "protocol" character varying,
                "port" integer NOT NULL,
                "version" character varying,
                "extra" json,
                "notes" character varying,
                "created_at" TIMESTAMP NOT NULL DEFAULT now(),
                "updated_at" TIMESTAMP NOT NULL DEFAULT now(),
                "deleted_at" TIMESTAMP,
                "target_id" uuid,
                CONSTRAINT "PK_ba2d347a3168a296416c6c5ccb2" PRIMARY KEY ("id")
            );
            COMMENT ON COLUMN "services"."extra" IS 'Extra data from scanner';
            COMMENT ON COLUMN "services"."notes" IS 'User notes for this service'
        `);
    await queryRunner.query(`
            CREATE TABLE "issues" (
                "id" uuid NOT NULL DEFAULT uuid_generate_v4(),
                "name" character varying NOT NULL,
                "description" character varying NOT NULL,
                "cve" character varying,
                "cvss" character varying,
                "extra" json,
                "notes" character varying,
                "last_verified_at" TIMESTAMP,
                "notified_at" TIMESTAMP,
                "resolved_at" TIMESTAMP,
                "created_at" TIMESTAMP NOT NULL DEFAULT now(),
                "updated_at" TIMESTAMP NOT NULL DEFAULT now(),
                "deleted_at" TIMESTAMP,
                "service_id" uuid,
                CONSTRAINT "PK_9d8ecbbeff46229c700f0449257" PRIMARY KEY ("id")
            );
            COMMENT ON COLUMN "issues"."name" IS 'Issue name';
            COMMENT ON COLUMN "issues"."description" IS 'Issue basic description';
            COMMENT ON COLUMN "issues"."cve" IS 'CVE identifier of the issue';
            COMMENT ON COLUMN "issues"."cvss" IS 'CVSS 3.0 vector';
            COMMENT ON COLUMN "issues"."extra" IS 'Extra data from scanner';
            COMMENT ON COLUMN "issues"."notes" IS 'User notes for this issue'
        `);
    await queryRunner.query(`
            CREATE TABLE "modules" (
                "id" uuid NOT NULL DEFAULT uuid_generate_v4(),
                "uid" character varying NOT NULL,
                "name" character varying NOT NULL,
                "description" character varying NOT NULL,
                "nickname" character varying,
                "notes" character varying,
                "status_code" character varying,
                "status_message" character varying,
                "status_last_update" TIMESTAMP,
                "status_expiry" integer NOT NULL,
                "configurations" jsonb NOT NULL,
                "created_at" TIMESTAMP NOT NULL DEFAULT now(),
                "updated_at" TIMESTAMP NOT NULL DEFAULT now(),
                "deleted_at" TIMESTAMP,
                CONSTRAINT "UQ_f96a68570f29f80339d956131f7" UNIQUE ("uid"),
                CONSTRAINT "PK_7dbefd488bd96c5bf31f0ce0c95" PRIMARY KEY ("id")
            );
            COMMENT ON COLUMN "modules"."uid" IS 'Unique identifier generated by the module';
            COMMENT ON COLUMN "modules"."name" IS 'Module name';
            COMMENT ON COLUMN "modules"."description" IS 'Module description';
            COMMENT ON COLUMN "modules"."nickname" IS 'Module nickname';
            COMMENT ON COLUMN "modules"."notes" IS 'User notes for this module';
            COMMENT ON COLUMN "modules"."status_code" IS 'Module status';
            COMMENT ON COLUMN "modules"."status_message" IS 'Module status message';
            COMMENT ON COLUMN "modules"."status_last_update" IS 'Last module status update timestamp';
            COMMENT ON COLUMN "modules"."status_expiry" IS 'Time period in seconds the status stays valid';
            COMMENT ON COLUMN "modules"."configurations" IS 'Configurations for this module'
        `);
    await queryRunner.query(`
            CREATE TABLE "ranges" (
                "id" uuid NOT NULL DEFAULT uuid_generate_v4(),
                "name" character varying NOT NULL,
                "description" character varying,
                "start_ip" character varying NOT NULL,
                "end_ip" character varying NOT NULL,
                "notes" character varying,
                "created_at" TIMESTAMP NOT NULL DEFAULT now(),
                "updated_at" TIMESTAMP NOT NULL DEFAULT now(),
                "deleted_at" TIMESTAMP,
                "target_id" uuid,
                CONSTRAINT "PK_cc6bfd0d09dda9fa9158cadb6c5" PRIMARY KEY ("id")
            );
            COMMENT ON COLUMN "ranges"."notes" IS 'User notes for this ip range'
        `);
    await queryRunner.query(`
            ALTER TABLE "users" DROP CONSTRAINT "UQ_97672ac88f789774dd47f7c8be3"
        `);
    await queryRunner.query(`
            ALTER TABLE "users" DROP COLUMN "email"
        `);
    await queryRunner.query(`
            ALTER TABLE "users"
            ADD "email" character varying NOT NULL
        `);
    await queryRunner.query(`
            ALTER TABLE "users"
            ADD CONSTRAINT "UQ_97672ac88f789774dd47f7c8be3" UNIQUE ("email")
        `);
    await queryRunner.query(`
            ALTER TABLE "users" DROP CONSTRAINT "UQ_fe0bb3f6520ee0469504521e710"
        `);
    await queryRunner.query(`
            ALTER TABLE "users" DROP COLUMN "username"
        `);
    await queryRunner.query(`
            ALTER TABLE "users"
            ADD "username" character varying
        `);
    await queryRunner.query(`
            ALTER TABLE "users"
            ADD CONSTRAINT "UQ_fe0bb3f6520ee0469504521e710" UNIQUE ("username")
        `);
    await queryRunner.query(`
            ALTER TABLE "users" DROP COLUMN "name"
        `);
    await queryRunner.query(`
            ALTER TABLE "users"
            ADD "name" character varying
        `);
    await queryRunner.query(`
            ALTER TABLE "services"
            ADD CONSTRAINT "FK_d44f4233e0075b167c9b5976b60" FOREIGN KEY ("target_id") REFERENCES "targets"("id") ON DELETE NO ACTION ON UPDATE NO ACTION
        `);
    await queryRunner.query(`
            ALTER TABLE "issues"
            ADD CONSTRAINT "FK_b0fb86b8df2a789f6d285b76f76" FOREIGN KEY ("service_id") REFERENCES "services"("id") ON DELETE NO ACTION ON UPDATE NO ACTION
        `);
    await queryRunner.query(`
            ALTER TABLE "ranges"
            ADD CONSTRAINT "FK_5fede7a5b88d0ae334adc7157f8" FOREIGN KEY ("target_id") REFERENCES "targets"("id") ON DELETE NO ACTION ON UPDATE NO ACTION
        `);
  }

  public async down(queryRunner: QueryRunner): Promise<void> {
    await queryRunner.query(`
            ALTER TABLE "ranges" DROP CONSTRAINT "FK_5fede7a5b88d0ae334adc7157f8"
        `);
    await queryRunner.query(`
            ALTER TABLE "issues" DROP CONSTRAINT "FK_b0fb86b8df2a789f6d285b76f76"
        `);
    await queryRunner.query(`
            ALTER TABLE "services" DROP CONSTRAINT "FK_d44f4233e0075b167c9b5976b60"
        `);
    await queryRunner.query(`
            ALTER TABLE "users" DROP COLUMN "name"
        `);
    await queryRunner.query(`
            ALTER TABLE "users"
            ADD "name" character varying(40)
        `);
    await queryRunner.query(`
            ALTER TABLE "users" DROP CONSTRAINT "UQ_fe0bb3f6520ee0469504521e710"
        `);
    await queryRunner.query(`
            ALTER TABLE "users" DROP COLUMN "username"
        `);
    await queryRunner.query(`
            ALTER TABLE "users"
            ADD "username" character varying(40)
        `);
    await queryRunner.query(`
            ALTER TABLE "users"
            ADD CONSTRAINT "UQ_fe0bb3f6520ee0469504521e710" UNIQUE ("username")
        `);
    await queryRunner.query(`
            ALTER TABLE "users" DROP CONSTRAINT "UQ_97672ac88f789774dd47f7c8be3"
        `);
    await queryRunner.query(`
            ALTER TABLE "users" DROP COLUMN "email"
        `);
    await queryRunner.query(`
            ALTER TABLE "users"
            ADD "email" character varying(100) NOT NULL
        `);
    await queryRunner.query(`
            ALTER TABLE "users"
            ADD CONSTRAINT "UQ_97672ac88f789774dd47f7c8be3" UNIQUE ("email")
        `);
    await queryRunner.query(`
            DROP TABLE "ranges"
        `);
    await queryRunner.query(`
            DROP TABLE "modules"
        `);
    await queryRunner.query(`
            DROP TABLE "issues"
        `);
    await queryRunner.query(`
            DROP TABLE "services"
        `);
    await queryRunner.query(`
            DROP TABLE "targets"
        `);
  }
}
